"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AdaptivePoller = void 0;
const timespan_1 = require("@/services/timespan");
class AdaptivePoller {
    initialDelay;
    logger;
    maxDelay;
    factor;
    resetPeriod;
    linearStep;
    currentTimeout = undefined;
    currentDelay;
    lastErrorTime = null;
    polling = false;
    operation = null;
    onError = null;
    onFail = null;
    get isPolling() {
        return this.polling;
    }
    /**
     * @param logger of the service that uses this polling
     * @param initialDelay initial interval between operations (in ms)
     * @param maxDelay maximum interval on errors (in ms)
     * @param factor multiplier for exponential interval increase
     * @param resetPeriod period (in ms) during which if no errors occur, the interval starts to decrease linearly
     * @param linearStep amount (in ms) by which the interval decreases after a successful operation, if resetPeriod has passed
     */
    constructor(initialDelay = timespan_1.TimeSpan.fromSeconds(1), logger = null, maxDelay = timespan_1.TimeSpan.fromMinutes(1), factor = 1.5, resetPeriod = timespan_1.TimeSpan.fromMinutes(5), linearStep = timespan_1.TimeSpan.fromSeconds(1)) {
        this.initialDelay = initialDelay;
        this.logger = logger;
        this.maxDelay = maxDelay;
        this.factor = factor;
        this.resetPeriod = resetPeriod;
        this.linearStep = linearStep;
        this.currentDelay = this.initialDelay.totalMilliseconds;
    }
    start(operation, onError = null, onFail = null) {
        if (this.polling)
            return;
        this.polling = true;
        this.operation = operation;
        this.onError = onError;
        this.onFail = onFail;
        this.currentDelay = this.initialDelay.totalMilliseconds;
        this.runPoll();
        this.logger?.info('AdaptivePolling started');
    }
    stop() {
        this.polling = false;
        clearTimeout(this.currentTimeout);
        this.logger?.info('AdaptivePolling stopped');
    }
    async runPoll() {
        if (!this.polling || !this.operation)
            return;
        try {
            await this.operation();
            const now = Date.now();
            if (this.lastErrorTime === null || (now - this.lastErrorTime) >= this.resetPeriod.totalMilliseconds) {
                this.currentDelay = Math.max(this.initialDelay.totalMilliseconds, this.currentDelay - this.linearStep.totalMilliseconds);
            }
            this.scheduleNextPoll();
        }
        catch (error) {
            this.lastErrorTime = Date.now();
            this.currentDelay = this.currentDelay * this.factor;
            const resultError = error instanceof Error ? error : new Error(JSON.stringify(error));
            if (this.currentDelay > this.maxDelay.totalMilliseconds) {
                this.currentDelay = this.maxDelay.totalMilliseconds;
                this.onFail?.(resultError);
            }
            else {
                this.onError?.(resultError);
            }
            this.scheduleNextPoll();
        }
    }
    scheduleNextPoll() {
        this.currentTimeout = setTimeout(() => this.runPoll(), this.currentDelay);
    }
}
exports.AdaptivePoller = AdaptivePoller;
