import { TimeSpan, ServiceLogger } from 'dash-kit';
/**
 * Handles polling with adaptive intervals, adjusting delays based on success or failure.
 */
export declare class AdaptivePoller {
    private readonly initialDelay;
    private readonly logger?;
    private readonly maxDelay;
    private readonly factor;
    private readonly resetPeriod;
    private readonly linearStep;
    private currentTimeout;
    private currentDelay;
    private lastErrorTime;
    private polling;
    private operation;
    private onError;
    private onFail;
    /**
     * Returns whether the polling operation is currently active.
     * @returns {boolean} True if polling is ongoing, false otherwise.
     */
    get isPolling(): boolean;
    /**
     * Creates an instance of the AdaptivePoller class with the specified configuration.
     * @param {TimeSpan} [initialDelay=TimeSpan.fromSeconds(1)] - The initial delay before the first operation.
     * @param {ServiceLogger} [logger] - A logger to log messages (optional).
     * @param {TimeSpan} [maxDelay=TimeSpan.fromMinutes(1)] - The maximum delay between operations on errors.
     * @param {number} [factor=1.5] - The multiplier for exponential interval increase on errors.
     * @param {TimeSpan} [resetPeriod=TimeSpan.fromMinutes(5)] - The period after which the interval starts to decrease linearly if no errors occur.
     * @param {TimeSpan} [linearStep=TimeSpan.fromSeconds(1)] - The step by which the interval decreases linearly on success.
     */
    constructor(initialDelay?: TimeSpan, logger?: ServiceLogger | undefined, maxDelay?: TimeSpan, factor?: number, resetPeriod?: TimeSpan, linearStep?: TimeSpan);
    /**
     * Starts the polling process.
     * The polling will continue until explicitly stopped.
     * @param {() => Promise<void>} operation - The operation to perform during each poll cycle.
     * @param {((error: Error) => void) | null} [onError=null] - A callback to handle errors during polling.
     * @param {((error: Error) => void) | null} [onFail=null] - A callback to handle failure events (when the interval exceeds maxDelay).
     */
    start(operation: () => Promise<void>, onError?: ((error: Error) => void) | null, onFail?: ((error: Error) => void) | null): void;
    /**
     * Stops the polling process.
     */
    stop(): void;
    private runPoll;
    private scheduleNextPoll;
}
