"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AdaptivePoller = void 0;
const dash_kit_1 = require("dash-kit");
/**
 * Handles polling with adaptive intervals, adjusting delays based on success or failure.
 */
class AdaptivePoller {
    initialDelay;
    logger;
    maxDelay;
    factor;
    resetPeriod;
    linearStep;
    currentTimeout = undefined;
    currentDelay;
    lastErrorTime = null;
    polling = false;
    operation = null;
    onError = null;
    onFail = null;
    /**
     * Returns whether the polling operation is currently active.
     * @returns {boolean} True if polling is ongoing, false otherwise.
     */
    get isPolling() {
        return this.polling;
    }
    /**
     * Creates an instance of the AdaptivePoller class with the specified configuration.
     * @param {TimeSpan} [initialDelay=TimeSpan.fromSeconds(1)] - The initial delay before the first operation.
     * @param {ServiceLogger} [logger] - A logger to log messages (optional).
     * @param {TimeSpan} [maxDelay=TimeSpan.fromMinutes(1)] - The maximum delay between operations on errors.
     * @param {number} [factor=1.5] - The multiplier for exponential interval increase on errors.
     * @param {TimeSpan} [resetPeriod=TimeSpan.fromMinutes(5)] - The period after which the interval starts to decrease linearly if no errors occur.
     * @param {TimeSpan} [linearStep=TimeSpan.fromSeconds(1)] - The step by which the interval decreases linearly on success.
     */
    constructor(initialDelay = dash_kit_1.TimeSpan.fromSeconds(1), logger, maxDelay = dash_kit_1.TimeSpan.fromMinutes(1), factor = 1.5, resetPeriod = dash_kit_1.TimeSpan.fromMinutes(5), linearStep = dash_kit_1.TimeSpan.fromSeconds(1)) {
        this.initialDelay = initialDelay;
        this.logger = logger;
        this.maxDelay = maxDelay;
        this.factor = factor;
        this.resetPeriod = resetPeriod;
        this.linearStep = linearStep;
        this.currentDelay = this.initialDelay.totalMilliseconds;
    }
    /**
     * Starts the polling process.
     * The polling will continue until explicitly stopped.
     * @param {() => Promise<void>} operation - The operation to perform during each poll cycle.
     * @param {((error: Error) => void) | null} [onError=null] - A callback to handle errors during polling.
     * @param {((error: Error) => void) | null} [onFail=null] - A callback to handle failure events (when the interval exceeds maxDelay).
     */
    start(operation, onError = null, onFail = null) {
        if (this.polling)
            return;
        this.polling = true;
        this.operation = operation;
        this.onError = onError;
        this.onFail = onFail;
        this.currentDelay = this.initialDelay.totalMilliseconds;
        this.runPoll();
        this.logger?.info('AdaptivePolling started');
    }
    /**
     * Stops the polling process.
     */
    stop() {
        this.polling = false;
        clearTimeout(this.currentTimeout);
        this.logger?.info('AdaptivePolling stopped');
    }
    async runPoll() {
        if (!this.polling || !this.operation)
            return;
        try {
            await this.operation();
            const now = Date.now();
            if (this.lastErrorTime === null || (now - this.lastErrorTime) >= this.resetPeriod.totalMilliseconds) {
                this.currentDelay = Math.max(this.initialDelay.totalMilliseconds, this.currentDelay - this.linearStep.totalMilliseconds);
            }
            this.scheduleNextPoll();
        }
        catch (error) {
            this.lastErrorTime = Date.now();
            this.currentDelay = this.currentDelay * this.factor;
            const resultError = error instanceof Error ? error : new Error(JSON.stringify(error));
            if (this.currentDelay > this.maxDelay.totalMilliseconds) {
                this.currentDelay = this.maxDelay.totalMilliseconds;
                this.onFail?.(resultError);
            }
            else {
                this.onError?.(resultError);
            }
            this.scheduleNextPoll();
        }
    }
    scheduleNextPoll() {
        this.currentTimeout = setTimeout(() => this.runPoll(), this.currentDelay);
    }
}
exports.AdaptivePoller = AdaptivePoller;
